@@ -27,8 +27,10 @@ extern DRV_CLASS dc_cpcemu;	/* CPCEMU DSK driver */
 extern DRV_CLASS dc_cpcext;	/* CPCEMU DSK driver: create in ext. format */
 extern DRV_CLASS dc_dqk;	/* Compressed CPCEMU driver */
 extern DRV_CLASS dc_posix;	/* POSIX driver */
+extern DRV_CLASS dc_nwasp;	/* NanoWasp driver */
 extern DRV_CLASS dc_myz80;	/* MYZ80 driver */
 extern DRV_CLASS dc_cfi;	/* CFI driver */
+extern DRV_CLASS dc_adisk;	/* APRIDISK driver */
 #ifdef LINUXFLOPPY
 extern DRV_CLASS dc_linux;	/* Linux driver */
 #endif
