@@ -24,10 +24,15 @@
 #include <string.h>
 #include <ctype.h>
 
+
 #include "config.h"
 #include "libdsk.h"
 #include "drv.h"
 
+#ifdef HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
 #ifndef DISABLE_FLOPPY
 # ifdef HAVE_LINUX_FD_H
 #  include "linux/fd.h"
