@@ -41,20 +41,37 @@ static dsk_format_t format = -1;
 static	char *intyp = NULL, *outtyp = NULL;
 static	char *incomp = NULL, *outcomp = NULL;
 static	int inside = -1, outside = -1;
+static  int idstep =  0, odstep  =  0;
+static  int retries = 1;
 
 int do_copy(char *infile, char *outfile);
 
+static void report(const char *s)
+{
+        printf("%s\r", s);
+        fflush(stdout);
+}
+
+static void report_end()
+{
+        printf("\r%-79.79s\r", "");
+        fflush(stdout);
+}
+
 
 int help(int argc, char **argv)
 {
 	fprintf(stderr, "Syntax: \n"
-                       "      %s in-image out-image { options }\n",
+                       "      %s {options} in-image out-image\n",
 			AV0);
 	fprintf(stderr,"\nOptions are:\n"
 		       "-itype <type>   type of input disc image\n"
                        "-otype <type>   type of output disc image\n"
                        "-iside <side>   Force side 0 or side 1 of input\n"
                        "-oside <side>   Force side 0 or side 1 of output\n"
+		       "-retry <count>  Set number of retries on error\n"
+		       "-idstep         Double-step when reading\n"
+		       "-odstep         Double-step when writing\n"
                        "-md3            Defeat MicroDesign 3 copy protection\n"
 		       "-logical        Rearrange tracks in logical order\n"
                        "                (Only useful when out-image type is 'raw' and reading discs\n"
@@ -64,9 +81,9 @@ int help(int argc, char **argv)
 		               "\nDefault out-image type is DSK.\n\n");
 		
 	fprintf(stderr, "eg: %s /dev/fd0 myfile1.DSK\n"
-                        "    %s /dev/fd0 myfile2.DSK -side 1\n" 
-                        "    %s /dev/fd0 md3boot.dsk -md3\n" 
-                        "    %s myfile.DSK /dev/fd0 -otype floppy\n", 
+                        "    %s -side 1 /dev/fd0 myfile2.DSK\n" 
+                        "    %s -md3 /dev/fd0 md3boot.dsk\n" 
+                        "    %s -otype floppy myfile.DSK /dev/fd0\n", 
 			AV0, AV0, AV0, AV0);
 	valid_formats();
 	return 1;
@@ -79,17 +96,26 @@ int main(int argc, char **argv)
 	if (find_arg("--version", argc, argv) > 0) return version(); 
 	if (argc < 3) return help(argc, argv);
 	if (find_arg("--help",    argc, argv) > 0) return help(argc, argv);
-	intyp     = check_type("-itype", argc, argv); 
-        outtyp    = check_type("-otype", argc, argv);
-	incomp    = check_type("-icomp", argc, argv); 
-        outcomp   = check_type("-ocomp", argc, argv);
-        inside    = check_forcehead("-iside", argc, argv);
-        outside   = check_forcehead("-oside", argc, argv);
-	if (find_arg("-md3", argc, argv) > 0) md3 = 1;
-	if (find_arg("-logical", argc, argv) > 0) logical = 1;
-	if (!outtyp) outtyp = "dsk";
-        format    = check_format("-format", argc, argv);
 
+	ignore_arg("-type", 2, &argc, argv);
+	ignore_arg("-side", 2, &argc, argv);
+	ignore_arg("-dstep", 2, &argc, argv);
+	ignore_arg("-comp", 2, &argc, argv);
+
+	intyp     = check_type("-itype", &argc, argv); 
+        outtyp    = check_type("-otype", &argc, argv);
+	incomp    = check_type("-icomp", &argc, argv); 
+        outcomp   = check_type("-ocomp", &argc, argv);
+        inside    = check_forcehead("-iside", &argc, argv);
+        outside   = check_forcehead("-oside", &argc, argv);
+	retries   = check_retry("-retry", &argc, argv);
+	if (present_arg("-idstep", &argc, argv)) idstep = 1;
+	if (present_arg("-odstep", &argc, argv)) odstep = 1;
+	if (present_arg("-md3", &argc, argv)) md3 = 1;
+	if (present_arg("-logical", &argc, argv)) logical = 1;
+	if (!outtyp) outtyp = "dsk";
+        format    = check_format("-format", &argc, argv);
+	args_complete(&argc, argv);
 	return do_copy(argv[1], argv[2]);
 }
 
@@ -103,13 +129,20 @@ int do_copy(char *infile, char *outfile)
 	dsk_phead_t head;
 	dsk_psect_t sec;
 	char *buf = NULL;
+	char *cmt = NULL;
 	DSK_GEOMETRY dg;
 	char *op = "Opening";
 
+        dsk_reportfunc_set(report, report_end);
+
 	        e = dsk_open (&indr,  infile,  intyp, incomp);
-	if (!e) e = dsk_set_forcehead(indr, inside);
+	if (!e) e = dsk_set_retry(indr, retries);
+	if (!e && inside >= 0) e = dsk_set_option(indr, "HEAD", inside);
 	if (!e) e = dsk_creat(&outdr, outfile, outtyp, outcomp);
-	if (!e) e = dsk_set_forcehead(outdr, outside);
+	if (!e && outside >= 0) e = dsk_set_option(outdr, "HEAD", outside);
+	if (!e && idstep) e = dsk_set_option(indr, "DOUBLESTEP", 1);
+	if (!e && odstep) e = dsk_set_option(outdr, "DOUBLESTEP", 1);
+	if (!e) e = dsk_set_retry(outdr, retries);
 	if (format == -1)
 	{
 		op = "Identifying disc";
@@ -123,6 +156,9 @@ int do_copy(char *infile, char *outfile)
 	}
 	if (!e)
 	{
+		/* Copy comment, if any */
+		dsk_get_comment(indr, &cmt);
+		dsk_set_comment(outdr, cmt);
 		printf("Input driver: %s\nOutput driver:%s\n%s",
 			dsk_drvdesc(indr), dsk_drvdesc(outdr),
 			logical ? "[tracks rearranged]\n" : "");
